# ---------------------------------------------------------------------------
# LI_Terr_Model.py
# Created John Hak
# NatureServe
# www.NatureServe.org
#This vesion keeps the weighted values of the input disturbance in the final product model.
# ---------------------------------------------------------------------------
# "C:\projects\BlueMoon\Landscape_Condition\Quispicanchi\Input\Landscape_Condition.mdb\testWeighted_Model" "1" "C:\Projects\BlueMoon\Quispicanchi\Quispicanchi_bnd.shp" "500" "C:\temp\test_lc1"
# "D:\Projects\national_condition\National Landscape Condition.mdb\tblWeighted_Distance_West" "1" "D:\Projects\national_condition\Pacific_NW_100km_clip.shp" "90" "C:\temp\test_lc1"
# Import system modules
import sys, string, os, glob, math, traceback, time, arcpy
from arcpy import env
from arcpy.sa import *
sa = arcpy.sa
#change these as needed for system defaults
tempFilesOutputPath = "C:\\temp\\jch"
#vistaToolboxPath = "C:\\Program Files (x86)\\NatureServe\\Vista\\Resources\\Tools\\GeoProcessingTools\\RasterStatistics.tbx"
#vistaToolboxPath = "C:\\Program Files\\vista\\source\\VistaResources\\Resources\\Tools\\GeoProcessingTools\\RasterStatistics.tbx"


# Check arg count
if (len(sys.argv) < 5):
  print "Not enough arguments."
  sys.exit(-1)

#Inputs - sys.argv[0] is py script
tblInputRasters = sys.argv[1]
#Intensity Boost
inten_boost = sys.argv[2]
#Mask shapefile path
mask = sys.argv[3]
#Cellsize float
v30i = float(sys.argv[4])
#Grid output path
fin_cond_path = sys.argv[5]
#Temporary output directory
if (len(sys.argv) > 6):
  if (sys.argv[6] != ""):
    tempFilesOutputPath = sys.argv[6]
# Load required toolboxes...
if (len(sys.argv) > 7):
  if (sys.argv[7] != ""):
    vistaToolboxPath = sys.argv[7]

env.overwriteOutput = 1
# env.workspace = arcpy.GetParameterAsText(0)
env.workspace = sys.argv[1]
# Setup logging
# Final Condition Model
fin_cond_dir = os.path.dirname(fin_cond_path)

if not os.path.exists(fin_cond_dir):
  os.makedirs(fin_cond_dir) # ensure the log directory exists.

GP = None

logfile = open(fin_cond_dir + os.sep + "python_run_log.txt", "a+")

def log(x, error=False):
  logfile.write(x)
  logfile.write("\n")
  logfile.flush()
  print(x)
  if (GP != None):
    if (error):
      arcpy.AddError(x)
    else:
      arcpy.AddMessage(x)

def error_handler(tp, vl, tb):
  errinf = "\n".join(traceback.format_exception(tp,vl,tb))
  log("ERROR", True)
  log(errinf, True)
  sys.exit(-1)

sys.excepthook = error_handler

log("--------------------------------\n")
log("Invoked: Quoted invocation is: ")
log(sys.executable + " " + string.join(map(lambda v: ("\"" + v + "\""), sys.argv), " "))
log("\n")


# Required log: Vista checks for this in the output.
#  If absent, links to "problem w/ python"
log("Python_Got_GpDispatch")

product = "ArcView"
#product = "ArcInfo"

arcpy.CheckProduct(product)
arcpy.SetProduct(product)

# Required log: If this is absent, Vista gives licensing-specific error
log("Python_Got_ArcGisLicense")

arcpy.CheckOutExtension("Spatial")
log("Checked out spatial extension")

# Load required toolboxes...
arcpy.ImportToolbox(vistaToolboxPath, "vistaRaster")
#log("Set Tool Box")

# Local variables...
# Set Input GIS Layers

#Set Start Time
start_time = time.time()
log("Start Time - " + str(start_time))

#Input ArcMap table that tracks input rasters for calculating index
log("Table: " + tblInputRasters)

#Intensity Boost
log("Intensity boost: " + str(inten_boost))

# Mask
if arcpy.Exists(mask):
    desc = arcpy.Describe(mask)
    exb = desc.Extent
    ex = "\"" + str(exb) + "\""
    print ex
else :
  desc = False;

# Set Cell size - Final set from Vista
v30i = float(sys.argv[4])
log("Cell size: " + str(v30i))

arcpy.OverwriteOutput = 1

counter = 0

rows = arcpy.SearchCursor(tblInputRasters)

fields = arcpy.ListFields(tblInputRasters)


if not os.path.exists(tempFilesOutputPath):
  os.makedirs(tempFilesOutputPath) # ensure the tmp output directory exists.

out_Workspace = tempFilesOutputPath
log("out_Workspace: " + out_Workspace)

arcpy.env.workspace = out_Workspace
print arcpy.env.workspace


site_tmp = out_Workspace + "\\" + "site_tmp"
from_tmp = out_Workspace + "\\" + "from_tmp"
#from_tmp1 = "c:/temp/from_test"
temp_wgh = out_Workspace + "\\" + "temp_wgh"
wt_temp = out_Workspace + "\\" + "wt_temp"
fc_temp  = out_Workspace + "\\" + "base_in"
base_max2 = out_Workspace + "\\" + "base_max2"
from_max = out_Workspace + "\\" + "from_max"
rowcnt = 0
rownum = int(arcpy.GetCount_management(tblInputRasters).getOutput(0))
for row in rows:
  rowcnt = rowcnt + 1
  fcs = row.Raster
  log("!!!!!New row (fcs) " + str(rowcnt) + "/" + str(rownum))
  arcpy.AddMessage("Processing - " + str(rowcnt) + "/" + str(rownum))
  fc = str(fcs)
  fci = str(fcs)
  print fci
  descRaster = arcpy.Describe(fci)
  rasterCellSize = descRaster.MeanCellHeight
  arcpy.env.cellSize = rasterCellSize
  cellRatio = int(v30i) / int(rasterCellSize)
  log ("Cell Ratio = " + str(cellRatio))
  if (cellRatio >= 2):
    arcpy.env.extent = desc.Extent
    cellRatio2 = cellRatio - 1
    inTrueRaster = 1
    arcpy.Resample_management(fci, fc_temp, str(v30i), "MAJORITY")
    fc = fc_temp
  else:
    fc_temp = fci
  ifc = row.inFrom
  ifcd = 1.0 - float(ifc)
  ifrom = float(ifc)
  ip = float(row.inPoint)
  ifcxip = ifc * ip
  arcpy.AddMessage("Input - " + fcs + " - Impact:" + str(ip) + " - Decay:" + str(ifc))
  # Skip the row if ifcxip is negative or zero
  if (ifcxip <= 0):
    continue

  # Run Model
  # ---
  ip_inv = 1 - ip
  inten_point = ip_inv
  dist = 100
  v30d =  float(dist) / float(v30i)
  div1 = 1.0
  #div = "2.0"
  div = 2.0
  nul = "0.0"
  if_inv = 1 - ifrom
  inten_from = ifrom

  arcpy.env.cellSize = v30i
  if desc:
    arcpy.env.extent = desc.Extent
  else:
    arcpy.env.extent = arcpy.Describe(fc).Extent

  if arcpy.Exists(mask):
    arcpy.env.mask = mask

  log(fc)
  inPathRaster = os.path.basename(fc)
  log("inPathRaster: " + inPathRaster)
  fc1 = ApplyEnvironment(fc)
  base1_eu = EucDistance(fc1, "", float(v30i), "") #eucdist_rv_n1
  log("(base1_eu)" + str(base1_eu))
  if arcpy.Exists(mask):
    base1_euc = ExtractByMask(base1_eu, mask)
  else:
    base1_euc = base1_eu
  log("Compelete Distance Measure")
  #base_euc3 = Divide(base1_euc, float(v30i)) #divid_ras
  base_euc3 = base1_euc
  log("(base_euc3)" + str(base_euc3))

  # Modify LessThan to a con statement
  base_max = arcpy.sa.Con(base_euc3, div1, base_euc3, "VALUE < 1") #con_divid_ra1
  log("(base_max)" + str(base_max))

  base_max.save(base_max2)
  log("(base_max2)" + str(base_max2))


  #*****************************************************
  #Weights
  fromRatio = float(v30i / 2)
  if ifrom >= fromRatio:
    log("Applying Intensity of Distance From Site")
    ifc_half = ifc / 2
    ifc_20per = ifc * 0.25
    pwr = (-0.25 * math.log(ip) - 0.0000000000000008)
    #from_max3 = arcpy.gp.RasterCalculator_sa(1/(1+(sa.Exp(-2*((base_max2-(ifc/2))/(ifc*0.2))))))
    #from_wgh = (1/(1+(sa.Exp(-2*((base_max2-ifc_half)/ifc_20per)))))
    #Adjust Curve to equal IP + Sigmoidal Curve (from_max3)
    base1 = Minus(base_max2, ifc_half)
    base2 = Divide(base1, ifc_20per)
    base3 = Times(base2, -2)
    base4 = sa.Exp(base3)
    base5 = Plus(1, base4)
    base6 = Divide(1, base5)
    from_wgh = Power(base6, pwr)

    if arcpy.Exists(from_tmp):
      from_wgh2 = Times(from_tmp, from_wgh)#Multiplicative Model
      log("(from_wgh2)" + str(from_wgh2))
      log("del (from_tmp)" + str(from_tmp))
      #Modify to test fuzzysum
      tempIn = "C:/temp/jch/coninputs/lyr" + str(rowcnt)
      #arcpy.CopyRaster_management(from_wgh, tempIn)
      arcpy.Delete_management(from_tmp, "raster")
      from_wgh2.save(from_tmp)
      log("sav (from_tmp)" + str(from_tmp))
    else:
      #Modify to test fuzzysum
      tempIn = "C:/temp/jch/coninputs/lyr" + str(rowcnt)
      #arcpy.CopyRaster_management(from_wgh, tempIn)
      from_wgh.save(from_tmp)
      log("sav (from_tmp)" + str(from_tmp))


  if ifrom < fromRatio:
    log("Applying Intensity at Site Only")
    if arcpy.Exists(from_tmp):
      from_wgh = Con(base_max2, ip, div1, "VALUE = 1")
      log("(from_wgh)" + str(from_wgh))
      from_wgh2 = Times(from_tmp, from_wgh)#Multiplicative Model
      log("(from_wgh2)" + str(from_wgh2))
      log("del (from_tmp)" + str(from_tmp))
      #Modify to test fuzzysum
      tempIn = "C:/temp/jch/coninputs/lyr" + str(rowcnt)
      #arcpy.CopyRaster_management(from_wgh, tempIn)
      arcpy.Delete_management(from_tmp, "raster")
      from_wgh2.save(from_tmp)
      log("sav (from_tmp)" + str(from_tmp))
    else:
      from_wgh2 = Con(base_max2, ip, div1, 'VALUE = 1')
      #Modify to test fuzzysum
      tempIn = "C:/temp/jch/coninputs/lyr" + str(rowcnt)
      #arcpy.CopyRaster_management(from_wgh, tempIn)
      from_wgh2.save(from_tmp)
      log("sav (from_tmp)" + str(from_tmp))

#abreak
log("Final Model")
arcpy.AddMessage("Processing Final Model")
count = 0

log("ex (from_tmp)" + str(from_tmp))
wt_fin = from_tmp
if float(str(inten_boost)) != 1.0:
  temp_pow =  Power(wt_fin, float(str(inten_boost)))
  log("(temp_pow)" + str(temp_pow))
  arcpy.CopyRaster_management(temp_pow, fin_cond_path)
  arcpy.CalculateStatistics_management(fin_cond_path, "1", "1", "", "SKIP_EXISTING")

else:
  arcpy.CopyRaster_management(from_tmp, fin_cond_path)
  arcpy.CalculateStatistics_management(fin_cond_path, "1", "1", "", "SKIP_EXISTING")

#Clean up temp files
rasterList = arcpy.ListRasters("*",)
for raster in rasterList:
    arcpy.Delete_management(raster, "raster")
    log("Deleted" + str(raster))
elapsed_time = (time.time() - start_time) / 3600
log("Model Complete - Saved to final destination - Run Time = " + str(elapsed_time))
arcpy.CheckInExtension("Spatial")

